@@ -1,5 +1,5 @@
-"""API for working with Nvim buffers."""
-from .common import Remote, RemoteMap
+"""API for working with a Nvim Buffer."""
+from .common import Remote
 from ..compat import IS_PYTHON3
 
 
@@ -14,22 +14,11 @@ class Buffer(Remote):
 
     """A remote Nvim buffer."""
 
-    def __init__(self, session, code_data):
-        """Initialize from session and code_data immutable object.
-
-        The `code_data` contains serialization information required for
-        msgpack-rpc calls. It must be immutable for Buffer equality to work.
-        """
-        self._session = session
-        self.code_data = code_data
-        self.vars = RemoteMap(session, 'buffer_get_var', 'buffer_set_var',
-                              self)
-        self.options = RemoteMap(session, 'buffer_get_option',
-                                 'buffer_set_option', self)
+    _api_prefix = "buffer_"
 
     def __len__(self):
         """Return the number of lines contained in a Buffer."""
-        return self._session.request('buffer_line_count', self)
+        return self.request('buffer_line_count')
 
     def __getitem__(self, idx):
         """Get a buffer line or slice by integer index.
@@ -115,13 +104,13 @@ class Buffer(Remote):
 
     def append(self, lines, index=-1):
         """Append a string or list of lines to the buffer."""
-        if isinstance(lines, basestring):
+        if isinstance(lines, (basestring, bytes)):
             lines = [lines]
         return self._session.request('buffer_insert', self, index, lines)
 
     def mark(self, name):
         """Return (row, col) tuple for a named mark."""
-        return self._session.request('buffer_get_mark', self, name)
+        return self.request('buffer_get_mark', name)
 
     def range(self, start, end):
         """Return a `Range` object, which represents part of the Buffer."""
@@ -132,44 +121,43 @@ class Buffer(Remote):
         """Add a highlight to the buffer."""
         if async is None:
             async = (src_id != 0)
-        return self._session.request('buffer_add_highlight', self, src_id,
-                                     hl_group, line, col_start,
-                                     col_end, async=async)
+        return self.request('buffer_add_highlight', src_id, hl_group,
+                            line, col_start, col_end, async=async)
 
     def clear_highlight(self, src_id, line_start=0, line_end=-1, async=True):
-        """clear highlights from the buffer."""
-        self._session.request('buffer_clear_highlight', self, src_id,
-                              line_start, line_end, async=async)
+        """Clear highlights from the buffer."""
+        self.request('buffer_clear_highlight', src_id,
+                     line_start, line_end, async=async)
 
     @property
     def name(self):
         """Get the buffer name."""
-        return self._session.request('buffer_get_name', self)
+        return self.request('buffer_get_name')
 
     @name.setter
     def name(self, value):
         """Set the buffer name. BufFilePre/BufFilePost are triggered."""
-        return self._session.request('buffer_set_name', self, value)
+        return self.request('buffer_set_name', value)
 
     @property
     def valid(self):
         """Return True if the buffer still exists."""
-        return self._session.request('buffer_is_valid', self)
+        return self.request('buffer_is_valid')
 
     @property
     def number(self):
         """Get the buffer number."""
-        return self._session.request('buffer_get_number', self)
+        return self.request('buffer_get_number')
 
 
 class Range(object):
     def __init__(self, buffer, start, end):
         self._buffer = buffer
         self.start = start - 1
-        self.end = end
+        self.end = end - 1
 
     def __len__(self):
-        return self.end - self.start
+        return self.end - self.start + 1
 
     def __getitem__(self, idx):
         if not isinstance(idx, slice):
@@ -179,7 +167,7 @@ class Range(object):
         if start is None:
             start = self.start
         if end is None:
-            end = self.end
+            end = self.end + 1
         return self._buffer[start:end]
 
     def __setitem__(self, idx, lines):
@@ -191,26 +179,26 @@ class Range(object):
         if start is None:
             start = self.start
         if end is None:
-            end = self.end
+            end = self.end + 1
         self._buffer[start:end] = lines
 
     def __iter__(self):
-        for i in range(self.start, self.end):
+        for i in range(self.start, self.end + 1):
             yield self._buffer[i]
 
     def append(self, lines, i=None):
         i = self._normalize_index(i)
         if i is None:
-            i = self.end
+            i = self.end + 1
         self._buffer.append(lines, i)
 
     def _normalize_index(self, index):
         if index is None:
             return None
         if index < 0:
-            index = self.end - 1
+            index = self.end
         else:
             index += self.start
-            if index >= self.end:
-                index = self.end - 1
+            if index > self.end:
+                index = self.end
         return index
